#pragma once

#include "scenes/SceneMonologue.hpp"
#include "games/vn/events/VNDummyEvent.hpp"
#include "games/vn/events/VNSetDefaultFontEvent.hpp"
#include "games/vn/events/VNTextEvent.hpp"
#include "games/vn/events/VNChoiceEvent.hpp"
#include "games/vn/events/VNIfEvent.hpp"
#include "games/vn/events/VNSceneChangeEvent.hpp"
#include "games/vn/events/VNChoiceSetEvent.hpp"
#include "vnscenes/ScenePrologue9.hpp"

namespace Dawn {
  class ScenePrologue8CustomEventForChoiceSet : public VNChoiceEvent {
    protected:
      void onStart() override {
        auto scene = dynamic_cast<ScenePrologue8*>this->manager->getScene();
        assertNotNull(scene, "ScenePrologue8CustomEventForChoiceSet - Scene is null?");

        this->choices = scene->remainingChoices;
        this->key = "killer";
        this->text = "It is...";

        VNChoiceSetEvent::onStart();
      }
  };

  class ScenePrologue8CustomEventForAfterChoice : public VNEvent {
    protected:
      void onStart() override {
        auto prev = dynamic_cast<ScenePrologue8CustomEventForChoiceSet*>(this->previous);
        assertNotNull(prev, "ScenePrologue8CustomEventForAfterChoice - Previous is null?");
        auto scene = dynamic_cast<ScenePrologue8*>this->manager->getScene();
        assertNotNull(scene, "ScenePrologue8CustomEventForAfterChoice - Scene is null?");

        auto key = prev->getChoiceKey();
        scene->remainingChoices.erase(key);

        this->next();
      }
  };

  class ScenePrologue8 : public SceneMonologue {
    public:
      std::map<std::string, std::string> remainingChoices;

      ScenePrologue8(DawnGame *game) : SceneMonologue(game) {
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto man = &this->game->assetManager;
        std::vector<Asset*> assets = SceneMonologue::getRequiredAssets();
        return assets;
      }
      
      void stage() override {
        SceneMonologue::stage();

        remainingChoices["ave"] = "Ave?";
        remainingChoices["ronin"] = "Ronin?";
        remainingChoices["craig"] = "Craig?";

        auto man = &this->game->assetManager;
        assertNotNull(vnManager, "VNSceneGenInit - VN Manager is null?");
        VNEvent *previous = vnManager->createEvent<VNDummyEvent>();
        auto eventStart = previous;

        auto event0 = vnManager->createEvent<VNSetDefaultFontEvent>();
        event0->font = "<font font=\"font_main\" size=\"32.0f\">{{ text }}</font>";

        auto event1 = vnManager->createEvent<ScenePrologue8CustomEventForChoiceSet>();
        auto event2 = vnManager->createEvent<ScenePrologue8CustomEventForAfterChoice>();
        auto sceneChange = vnManager->createEvent<VNSceneChangeEvent<ScenePrologue9>>();
        
        eventStart->then(event0);
        event0->then(event1);
        event1->then(event2);
        event2->then(sceneChange);
        vnManager->setEvent(eventStart);
      }
  };
}
